﻿using JetBrains.Annotations;
using UniRx;
using UnityEngine.Pool;

namespace Magify.Features
{
    internal class FeaturesUpdater : IInitializable, ICancelable
    {
        [NotNull]
        private FeaturesProvider _featuresProvider;
        [NotNull]
        private StoredAppFeaturesProvider _storedAppFeaturesProvider;
        [NotNull]
        private readonly GeneralPrefs _generalPrefs;
        [NotNull]
        private readonly CompositeDisposable _disposables = GenericPool<CompositeDisposable>.Get() ?? new CompositeDisposable();

        public bool IsDisposed { get; private set; }

        public FeaturesUpdater(
            [NotNull] FeaturesProvider featuresProvider,
            [NotNull] StoredAppFeaturesProvider storedAppFeaturesProvider,
            [NotNull] GeneralPrefs generalPrefs)
        {
            _featuresProvider = featuresProvider;
            _storedAppFeaturesProvider = storedAppFeaturesProvider;
            _generalPrefs = generalPrefs;
        }

        public void Dispose()
        {
            if (IsDisposed)
                return;
            IsDisposed = true;
            _disposables.Clear();
            GenericPool<CompositeDisposable>.Release(_disposables);
        }

        void IInitializable.Initialize()
        {
            if (IsDisposed)
            {
                throw new MagifyObjectDisposedException(nameof(FeaturesUpdater));
            }
            _generalPrefs.SubscriptionStatus
                .SkipLatestValueOnSubscribe()
                .Subscribe(UpdateFeaturesOnTrigger)
                .AddTo(_disposables);

            _generalPrefs.InAppStatus
                .SkipLatestValueOnSubscribe()
                .Subscribe(UpdateFeaturesOnTrigger)
                .AddTo(_disposables);

            _generalPrefs.AuthorizationStatus
                .SkipLatestValueOnSubscribe()
                .Subscribe(UpdateFeaturesOnTrigger)
                .AddTo(_disposables);

            _generalPrefs.ReferrerId
                .SkipLatestValueOnSubscribe()
                .Subscribe(UpdateFeaturesOnTrigger)
                .AddTo(_disposables);
        }

        public void OnProductPurchase()
        {
            UpdateFeaturesOnTrigger(0);
        }

        private void UpdateFeaturesOnTrigger<T>([CanBeNull] T _)
        {
            _featuresProvider.UpdateFeaturesOnTrigger();
            _storedAppFeaturesProvider.UpdateFeaturesOnTrigger();
        }
    }
}